"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = loadConfig;
exports.generateConfigTemplate = generateConfigTemplate;
exports.updateConfig = updateConfig;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const prompts_1 = require("@inquirer/prompts");
const logger_1 = require("./logger");
// Name of the configuration file
const CONFIG_FILE_NAME = 'op-sync.yaml';
/**
 * Loads and validates the configuration from the YAML file
 * @returns {Promise<Config>} A promise that resolves to the validated configuration object
 * @throws {Error} If the configuration file is missing or invalid
 */
async function loadConfig() {
    const configPath = path_1.default.join(process.cwd(), CONFIG_FILE_NAME);
    try {
        const fileContents = await promises_1.default.readFile(configPath, 'utf8');
        const config = js_yaml_1.default.load(fileContents);
        // Validate the configuration
        if (!config || typeof config !== 'object') {
            throw new Error(`Invalid ${CONFIG_FILE_NAME}: Configuration must be a YAML object`);
        }
        if (!config.token) {
            throw new Error(`Invalid ${CONFIG_FILE_NAME}: 'token' is missing`);
        }
        if (typeof config.token !== 'string' || config.token.trim() === '') {
            throw new Error(`Invalid ${CONFIG_FILE_NAME}: 'token' must be a non-empty string`);
        }
        if (config.token === '<replace_me_with_token>') {
            throw new Error(`${CONFIG_FILE_NAME}: Please replace the placeholder token with your actual 1Password service account token`);
        }
        // Set default values for optional fields
        config.storageMode = config.storageMode === 'combined' ? 'combined' : 'separate';
        config.projectPrefix = config.projectPrefix || '';
        return config;
    }
    catch (error) {
        if (error.code === 'ENOENT') {
            throw new Error(`${CONFIG_FILE_NAME} not found. Run 'init' command to create it.`);
        }
        throw error;
    }
}
/**
 * Generates a configuration template file
 * @returns {Promise<void>}
 */
async function generateConfigTemplate() {
    const configPath = path_1.default.join(process.cwd(), CONFIG_FILE_NAME);
    logger_1.logger.info(`Checking for config file at: ${configPath}`);
    try {
        await promises_1.default.access(configPath);
        logger_1.logger.info('File exists, checking contents...');
        const fileContents = await promises_1.default.readFile(configPath, 'utf8');
        logger_1.logger.info('File contents:');
        console.log(fileContents);
        const existingConfig = js_yaml_1.default.load(fileContents);
        logger_1.logger.info('Parsed config:');
        console.log(existingConfig);
        if (existingConfig && existingConfig.token) {
            logger_1.logger.success('Valid configuration found.');
            const answer = await (0, prompts_1.select)({
                message: `${CONFIG_FILE_NAME} already exists and appears to be valid. What would you like to do?`,
                choices: [
                    { value: 'keep', name: 'Keep existing file' },
                    { value: 'override', name: 'Override with new template' },
                ],
            });
            if (answer === 'keep') {
                logger_1.logger.info(`Keeping existing ${CONFIG_FILE_NAME} file.`);
                return;
            }
        }
    }
    catch (error) {
        if (error.code === 'ENOENT') {
            logger_1.logger.warning(`No existing ${CONFIG_FILE_NAME} file found. Creating a new one.`);
        }
        else {
            logger_1.logger.error(`Error checking file: ${error.message}`);
        }
    }
    // Prompt user for configuration details
    logger_1.logger.info('\nYou can provide the 1Password service account token now or leave it empty and fill it in later.');
    logger_1.logger.info('If left empty, a placeholder value will be used in the file.');
    const token = await (0, prompts_1.input)({
        message: 'Enter your 1Password service account token (or leave empty):',
    });
    const storageMode = await (0, prompts_1.select)({
        message: 'Choose the default storage mode for secrets:',
        choices: [
            { value: 'separate', name: 'As separate items for each secret' },
            { value: 'combined', name: 'As one item with fields for all secrets' },
        ],
    });
    const projectPrefix = await (0, prompts_1.input)({
        message: 'Enter a project prefix for 1Password items (or leave empty):',
    });
    // Generate the configuration template
    const template = `# 1Password configuration
token: ${token || '<replace_me_with_token>'}
storageMode: ${storageMode}
projectPrefix: ${projectPrefix || ''}
`;
    try {
        await promises_1.default.writeFile(configPath, template);
        logger_1.logger.success(`\n${CONFIG_FILE_NAME} has been created at ${configPath}`);
        if (!token) {
            logger_1.logger.warning('Please edit this file to add your 1Password service account token before using the tool.');
        }
    }
    catch (error) {
        logger_1.logger.error(`Failed to create ${CONFIG_FILE_NAME}: ${error.message}`);
    }
}
/**
 * Updates the configuration file with new values
 * @param {Partial<Config>} updates - Object containing the fields to update
 * @returns {Promise<void>}
 */
async function updateConfig(updates) {
    const configPath = path_1.default.join(process.cwd(), CONFIG_FILE_NAME);
    try {
        const currentConfig = await loadConfig();
        const updatedConfig = { ...currentConfig, ...updates };
        await promises_1.default.writeFile(configPath, js_yaml_1.default.dump(updatedConfig));
        logger_1.logger.success(`${CONFIG_FILE_NAME} has been updated.`);
    }
    catch (error) {
        logger_1.logger.error(`Failed to update ${CONFIG_FILE_NAME}: ${error.message}`);
    }
}
