"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkEnvFile = checkEnvFile;
exports.parseEnvSecrets = parseEnvSecrets;
exports.listEnvFiles = listEnvFiles;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const dotenv_1 = __importDefault(require("dotenv"));
/**
 * Checks if a specified environment file exists in the current working directory.
 *
 * @param {string} envFile - The name of the environment file to check.
 * @returns {Promise<boolean>} A promise that resolves to true if the file exists, false otherwise.
 */
async function checkEnvFile(envFile) {
    const envPath = path_1.default.join(process.cwd(), envFile);
    try {
        await promises_1.default.access(envPath);
        return true;
    }
    catch (error) {
        // If fs.access throws an error, the file doesn't exist
        return false;
    }
}
/**
 * Parses the contents of a specified environment file and returns its secrets as key-value pairs.
 *
 * @param {string} envFile - The name of the environment file to parse.
 * @returns {Promise<Record<string, string>>} A promise that resolves to an object containing the parsed environment variables.
 * @throws {Error} If the file cannot be read or parsed.
 */
async function parseEnvSecrets(envFile) {
    const envPath = path_1.default.join(process.cwd(), envFile);
    const envContent = await promises_1.default.readFile(envPath, 'utf-8');
    return dotenv_1.default.parse(envContent);
}
/**
 * Lists all environment files in the current working directory.
 * Environment files are identified by the '.env' extension or by matching the pattern '.*\.env'.
 *
 * @returns {Promise<string[]>} A promise that resolves to an array of environment file names.
 */
async function listEnvFiles() {
    const files = await promises_1.default.readdir(process.cwd());
    return files.filter(file => file.endsWith('.env') || file.match(/\..*\.env$/));
}
