"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = main;
const commander_1 = require("commander");
const envManager_1 = require("./envManager");
const onePasswordManager_1 = require("./onePasswordManager");
const syncManager_1 = require("./syncManager");
const utils_1 = require("./utils");
const configManager_1 = require("./configManager");
const logger_1 = require("./logger");
/**
 * Main function to set up and run the CLI application
 */
async function main() {
    const program = new commander_1.Command();
    // Set up the main program
    program
        .version('1.0.0')
        .description('Sync secrets between .env and 1Password vault')
        .option('-e, --env <environment>', 'Specify the environment (e.g., local, dev, prod)', 'default')
        .option('--op-token <token>', '1Password access token');
    // 'init' command
    program
        .command('init')
        .description('Generate a template op-sync.yaml file')
        .action(configManager_1.generateConfigTemplate);
    // 'sync' command
    program
        .command('sync')
        .description('Sync secrets between .env and 1Password')
        .action(async (options) => {
        try {
            // Select .env file
            const availableEnvFiles = await (0, envManager_1.listEnvFiles)();
            const selectedEnvFile = await (0, utils_1.selectEnvFile)(availableEnvFiles);
            // Initialize 1Password client
            logger_1.logger.info('Initializing 1Password SDK...');
            const client = await (0, onePasswordManager_1.initializeOnePassword)(program.opts());
            // Select 1Password vault
            logger_1.logger.info('Selecting vault...');
            const vault = await (0, onePasswordManager_1.selectVault)(client);
            // Determine sync direction
            const direction = await (0, utils_1.askSyncDirection)();
            if (direction === 'env-to-1password') {
                // Sync from .env to 1Password
                logger_1.logger.info('Syncing from .env to 1Password...');
                const secrets = await (0, envManager_1.parseEnvSecrets)(selectedEnvFile);
                await (0, syncManager_1.syncEnvToOnePassword)(client, vault, secrets);
            }
            else {
                // Sync from 1Password to .env
                logger_1.logger.info('Syncing from 1Password to .env...');
                await (0, syncManager_1.syncOnePasswordToEnv)(client, vault, selectedEnvFile);
            }
        }
        catch (error) {
            logger_1.logger.error(`An error occurred: ${error.message}`);
        }
    });
    // 'diff' command
    program
        .command('diff')
        .description('Show differences between .env and 1Password secrets')
        .action(async (options) => {
        try {
            // Select .env file
            const availableEnvFiles = await (0, envManager_1.listEnvFiles)();
            const selectedEnvFile = await (0, utils_1.selectEnvFile)(availableEnvFiles);
            // Parse .env secrets
            logger_1.logger.info('Parsing .env secrets...');
            const secrets = await (0, envManager_1.parseEnvSecrets)(selectedEnvFile);
            // Initialize 1Password client
            logger_1.logger.info('Initializing 1Password SDK...');
            const client = await (0, onePasswordManager_1.initializeOnePassword)(program.opts());
            // Select 1Password vault
            logger_1.logger.info('Selecting vault...');
            const vault = await (0, onePasswordManager_1.selectVault)(client);
            // Show differences
            logger_1.logger.info('Showing differences...');
            await (0, syncManager_1.showDifferences)(client, vault, secrets);
        }
        catch (error) {
            logger_1.logger.error(`An error occurred: ${error.message}`);
        }
    });
    // Parse command line arguments
    await program.parseAsync(process.argv);
}
